@@ -5,34 +5,38 @@ from . import ast
 from .unparse.maybeerror import MaybeError
 
 
-good, bad = MaybeError.pure, MaybeError.error
+good = MaybeError.pure
+
+def bad(**kwargs):
+    return MaybeError.error(kwargs)
 
 
 def buildLoop(loop):
-    pos, keys_1, vals_1 = loop['_state'], loop['keys'], loop['values']
-    
-    keys = [k['value'] for k in keys_1]
-    vals = [v['value'] for v in vals_1]
-
     # duplicate keys
-    key_check = set([])
-    for key in keys:
+    key_check = {}
+    for node in loop['keys']:
+        key = node['value']
         if key in key_check:
-            return bad(('loop: duplicate key', key, pos))
-        key_check.add(key)
-            
+            return bad(nodetype='loop', message='duplicate key', 
+                       key=key, first=key_check[key]['pos'], second=node['pos'])
+        key_check[key] = node
+
+    # can't simply pull the keys out of `key_check` b/c that wouldn't preserve order
+    keys = [k['value'] for k in loop['keys']]
+    vals = [v['value'] for v in loop['values']]
+    
     # no values
     if len(vals) == 0:
         return good(ast.Loop(keys, []))
 
     # values, but no keys -> throws ZeroDivisionError
     if len(keys) == 0:
-        return bad(('loop: keys but no values', pos))
+        return bad(nodetype='loop', message='values but no keys', position=loop['open']['pos'])
     
     # number of values okay?
     if len(vals) % len(keys) != 0:
-        return bad(('loop: number of values must be integer multiple of number of keys', 
-                    len(vals), len(keys), pos))
+        return bad(nodetype='loop', numkeys=len(keys), numvals=len(vals), position=loop['open']['pos'],
+                   message='number of values must be integer multiple of number of keys')
     
     rows, numKeys, valArr = [], len(keys), vals
     while len(valArr) > 0:
@@ -49,12 +53,15 @@ def buildSave(save):
       - duplicate datum keys
     '''
     loops, datums = [], {}
+    key_check = {}
 
     for d in save['datums']:
         key, value = d['key']['value'], d['value']['value']
         if datums.has_key(key):
-            return bad(('save: duplicate key', key, save['_state']))
+            return bad(nodetype='save', message='duplicate key',
+                       key=key, first=key_check[key], second=d['key']['pos'])
         datums[key] = value
+        key_check[key] = d['key']['pos']
     
     for loop in save['loops'] :
         l = buildLoop(loop)
@@ -68,15 +75,18 @@ def buildSave(save):
 
 def buildData(node):
     dataName, saves = node['open']['value'], node['saves']
-    mySaves = {}
+    mySaves, save_check = {}, {}
     for save in saves:
         if save['_name'] != 'save': 
             raise TypeError(('Data expects Saves', save))
-        if mySaves.has_key(save['open']['value']):
-            return bad(('data: duplicate save frame name', save['open']['value'], node['_state']))
+        name = save['open']['value']
+        if name in save_check:
+            return bad(nodetype='data', message='duplicate save frame name', 
+                       name=name, first=save_check[name], second=save['open']['pos'])
         s = buildSave(save)
         if s.status == 'success':
             mySaves[save['open']['value']] = s.value
+            save_check[name] = save['open']['pos']
         else:
             return s
     return good(ast.Data(dataName, mySaves))
